"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
class PathHelper {
    static getParentPath(childPath) {
        if (_.isEmpty(childPath)) {
            return null;
        }
        const parentPaths = PathHelper.pathToKeys(childPath);
        parentPaths.splice(-1);
        return !_.isEmpty(parentPaths)
            ? PathHelper.keysToPath(parentPaths)
            : "";
    }
    static isParentPath(inParentPath, inChildPath) {
        if (_.isEmpty(inParentPath)) {
            return !_.isEmpty(inChildPath);
        }
        const [parentPaths, childPaths] = [inParentPath, inChildPath]
            .map(path => !_.isEmpty(path) ? path : '')
            .map(path => _.trim(path, '.'))
            .map(path => path.split('.'));
        if (childPaths.length === parentPaths.length) {
            return false;
        }
        for (let i = 0; i < parentPaths.length; i++) {
            if (parentPaths[i] !== childPaths[i]) {
                return false;
            }
        }
        return true;
    }
    static applyToExpandedSubPaths(path, apply = _.identity) {
        const keys = PathHelper.pathToKeys(path);
        return keys.map((_, idx) => {
            const childKeys = [...keys].splice(0, idx + 1);
            const childPath = PathHelper.keysToPath(childKeys);
            return apply(childPath);
        });
    }
    static expandWildcardsInPath(path, data) {
        if (!path.includes(PathHelper.ARRAY_WILDCARD)) {
            return [path];
        }
        const [p, ...rest] = path.split(PathHelper.ARRAY_WILDCARD);
        const items = _.get(data, p, []);
        if (items.length === 0) {
            return [p];
        }
        return _.flatMap(items, (_, idx) => {
            const expandedPath = `${p}[${idx}]${rest.join(PathHelper.ARRAY_WILDCARD)}`;
            return PathHelper.expandWildcardsInPath(expandedPath, data);
        });
    }
    static reduceWildcardsInPaths(paths, apply, data = {}, initialValue) {
        return paths.reduce((acc, parentPath, index) => {
            const expandedWildcards = PathHelper.expandWildcardsInPath(parentPath, data);
            expandedWildcards.forEach((expandedPath) => {
                apply(acc, parentPath, expandedPath, index);
            });
            return acc;
        }, initialValue);
    }
    static reduceParentsToChildren(paths, apply, initialValue) {
        const subPaths = _(paths)
            .flatMap(path => PathHelper.applyToExpandedSubPaths(path))
            .map(PathHelper.pathToKeys)
            .sort((a, b) => a.length - b.length)
            .map(PathHelper.keysToPath)
            .uniq()
            .value();
        return subPaths.reduce((acc, childPath) => {
            apply(acc, PathHelper.getParentPath(childPath), childPath);
            return acc;
        }, initialValue);
    }
    static pathToKeys(path) {
        return _.trim(path, '.')
            .split(/\.|\[/)
            .map(x => _.trim(x, ']'));
    }
    static keysToPath(keys) {
        return (keys || []).reduce((str, value) => {
            if (str == null) {
                return value;
            }
            return !isNaN(value)
                ? `${str}[${value}]`
                : `${str}.${value}`;
        }, null);
    }
}
PathHelper.ARRAY_WILDCARD = '[*]';
exports.PathHelper = PathHelper;
//# sourceMappingURL=PathHelper.js.map