"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const ReadWriteObjectMapper_1 = require("./ReadWriteObjectMapper");
const REDUX_READ_WRITE_OBJECT_MAPPER_SET_ACTION = 'REDUX_READ_WRITE_OBJECT_MAPPER_SET_ACTION';
function buildReduxReadWriteObjectMapper(key, objectMapper, initialState = {}) {
    const actionKey = key.toUpperCase();
    function reducer(state = initialState, { type, value, path }) {
        switch (type) {
            case `${REDUX_READ_WRITE_OBJECT_MAPPER_SET_ACTION}_${actionKey}`:
                return ReadWriteObjectMapper_1.parseAndSetDataInObjectPurely(state, value, path, objectMapper);
        }
        return state;
    }
    function createSetAction(value, path) {
        return {
            type: `${REDUX_READ_WRITE_OBJECT_MAPPER_SET_ACTION}_${actionKey}`,
            value,
            path
        };
    }
    function connect(reduxConnect, stateSubPath) {
        return reduxConnect((state) => {
            const val = !_.isEmpty(stateSubPath)
                ? _.get(state, stateSubPath)
                : state;
            return val != null
                ? val
                : {};
        }, (dispatch) => ({
            dispatchSetAction: (value, path) => dispatch(createSetAction(value, path))
        }), (stateProps, { dispatchSetAction }, ownProps) => {
            return Object.assign({}, ownProps, { [key]: objectMapper
                    .map('', ReadWriteObjectMapper_1.buildReadWriteObjectMapper(dispatchSetAction))
                    .apply(stateProps) });
        });
    }
    return {
        reducer,
        connect
    };
}
exports.buildReduxReadWriteObjectMapper = buildReduxReadWriteObjectMapper;
//# sourceMappingURL=ReduxReadWriteObjectMapper.js.map